@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '0.81';
+$VERSION = '0.82';
 
 use base 'Exporter';
 
@@ -331,7 +331,7 @@ $builtin_types{gMonthDay} =
  };
 
 
-my $gYear = qr/^ $yearFrag \- $monthFrag $timezoneFrag? $/x;
+my $gYear = qr/^ $yearFrag $timezoneFrag? $/x;
 $builtin_types{gYear} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gYear }
@@ -446,31 +446,20 @@ $builtin_types{anyURI} =
  };
 
 
-sub _valid_qname($)
-{   my @ncnames = split /\:/, $_[0];
-    return 0 if @ncnames > 2;
-    _valid_ncname($_) || return 0 for @ncnames;
-    1;
-}
-
 $builtin_types{QName} =
  { parse   =>
      sub { my ($qname, $node) = @_;
+           $qname =~ s/\s//g;
            my $prefix = $qname =~ s/^([^:]*)\:// ? $1 : '';
 
-           length $prefix
-               or error __x"QNAME requires prefix at `{qname}'", qname=>$qname;
-
-           $node = $node->node if $node->isa('XML::Compile::Iterator');
-           my $ns = $node->lookupNamespaceURI($prefix)
-               or error __x"cannot find prefix `{prefix}' for QNAME `{qname}'"
-                     , prefix => $prefix, qname => $qname;
+           $node  = $node->node if $node->isa('XML::Compile::Iterator');
+           my $ns = $node->lookupNamespaceURI($prefix) || '';
            pack_type $ns, $qname;
          }
  , format  =>
     sub { my ($type, $trans) = @_;
           my ($ns, $local) = unpack_type $type;
-          $ns or return $local;
+          defined $ns or return $local;
 
           my $def = $trans->{$ns};
           if(!$def || !$def->{used})
@@ -478,8 +467,7 @@ $builtin_types{QName} =
           }
           "$def->{prefix}:$local";
         }
- , check   => \&_valid_qname
- , example => 'myns:name'
+ , example => 'myns:local'
  };
 
 
