@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema::XmlWriter;
 use vars '$VERSION';
-$VERSION = '0.81';
+$VERSION = '0.82';
 
 
 use strict;
@@ -124,7 +124,7 @@ sub element_wrapper
 {   my ($path, $args, $processor) = @_;
     sub { my ($doc, $data) = @_;
           UNIVERSAL::isa($doc, 'XML::LibXML::Document')
-              or error __x"first argument of writer must be an XML::LibXML::Document";
+              or error __x"first argument of call to writer must be an XML::LibXML::Document";
 
           my $top = $processor->(@_);
           $doc->indexElements;
